﻿--[[
********************************************************************************
RepBuddy Options (Ace3)
********************************************************************************
]]

----------------------------------------------------------------------------------------------------------------
-- Declarations
----------------------------------------------------------------------------------------------------------------
local RepBuddy = LibStub("AceAddon-3.0"):GetAddon("RepBuddy")
local L = LibStub("AceLocale-3.0"):GetLocale("RepBuddy", true)

----------------------------------------------------------------------------------------------------------------
-- Main Options
----------------------------------------------------------------------------------------------------------------
RepBuddyOptionsMain = {
    name = "RepBuddy",
    handler = RepBuddy,
    type = 'group',
  	args = {
		generaldesc = {
			order = 100,
			type = "header",
			name = L["General Options"],
		},
        
		outputdesc = {
			order = 200,
			type = "header",
			name = L["Output Options"],
		},
		

		enable = {
            order = 150,
			type = "toggle",
            name = L["Enabled"],
            desc = L["Enables/Disables RepBuddy"],
			get = function() return RepBuddy.db.profile.turnedOn end,
			set = "ToggleTurnedOn",
		},
		
        minimapIcon = {
				order = 151,
				type = "toggle",
				name = L["Show Minimap Icon"],
				desc = L["Show an Icon to open the config at the Minimap"],
				get = function() return not RepBuddy.db.profile.minimapIcon.hide end,
				set = "ToggleMiniMap"
			},		
		
        updatebar = {		
			order = 152,
            type = "toggle",
            name = L["Update Reputation Bar"],
            desc = L["Change reputation tracker bar to reflect current reputation changes. (Note: Behaves unpredictably when multiple factions are updated at once.)"],
            get = function() return RepBuddy.db.profile.changeBar end,
            set = "ToggleUpdateBar"
        },
        
        suggestreset = {		
			order = 153,
            type = "toggle",
            name = L["Suggest Reset"],
            desc = L["When entering an dungeon or raid for the first time during a game session, RepBuddy will ask if you'd like to reset the data. Handy if you want to track rep in a dungeon run or raid and don't want to forget to reset."],
            get = function() return RepBuddy.db.profile.suggestReset end,
            set = "ToggleSuggestReset"
        },
        
        reset = {
            order = 154,
			type = "execute",
            name = L["Reset RepBuddy"],
            desc = L["Resets Session Timer, Reputation Stats."],
			confirm = true,
			confirmText = L["Are you sure you want to reset RepBuddy?"],
            func = "ResetStats"
        },
        
        repwindow = {
			order = 155,
            type = "execute",
            name = L["View Blizz Rep Window"],
            desc = L["Opens the Blizzard reputation window."],
            func = function() ToggleCharacter("ReputationFrame") end,
        },
		chooseframe = {
			type = "select",
			order = 248,
            width = full,
			name = L["Choose Output Window:"],
			desc = L["Sets Window (Chatframe)that RepBuddy will output to."],
			values = function(info, value)
                chatframe_table = {}
                for i = 1, NUM_CHAT_WINDOWS do
                    --local name, _, _, _, _, _, _, _, _, _ = GetChatWindowInfo(i)
                    local name = GetChatWindowInfo(i)
                    if name ~= "" then
                        chatframe_table[i] = "Chatframe"..i..": "..name
                    end
                end
                return chatframe_table
            end,
			get = function() return RepBuddy.db.profile.chatFrameID end,
            set = function(info, value) 
                RepBuddy.db.profile.chatFrameID = value
                local name = GetChatWindowInfo(RepBuddy.db.profile.chatFrameID)
                chatframe_rb = _G["ChatFrame"..RepBuddy.db.profile.chatFrameID]
                RepBuddy:Print(L["Output to "].."|cff00ff00"..name.." (ChatFrame"..RepBuddy.db.profile.chatFrameID..")|r")
            end,
		},
		hideblizzrep = {		
			order = 249,
            type = "toggle",
            name = L["Hide Blizzard Updates"],
            desc = L["Turns off the standard Blizzard reputation updates in chatframe."],
            get = function() return RepBuddy.db.profile.hideBlizzRep end,
            set = "ToggleHideBlizzRep"
        },
        xtrastats = {
            order = 250,
			type = "toggle",
            name = L["Extra Stats"],
            desc = L["Toggles display of estimated time repetitions remaining."],
            get = function() return RepBuddy.db.profile.repinfo_verbose_display end,
            set = "ToggleXtraStats"
        },
		
		total = {
            order = 251,
			type = "toggle",
            name = L["Total Reputation"],
            desc = L["Toggles display of total reputation gained/lost during the session."],
            get = function() return RepBuddy.db.profile.sessionrep_display end,
            set = "ToggleTotalDisplay"
        },
		
		rph = {
            order = 252,
			type = "toggle",
            name = L["Rep/Hour (RPH)"],
            desc = L["Toggles display of session rep/hour."],
            get = function() return RepBuddy.db.profile.sessionrph_display end,
            set = "ToggleRPHDisplay"
        },
		
		elapsed = {
			order = 253,
            type = "toggle",
            name = L["Time Elapsed"],
            desc = L["Toggles display of session time elapsed."],
            get = function() return RepBuddy.db.profile.sessiontime_display end,
            set = "ToggleElapsedDisplay"
        },
        spacer1 = {
            order = 255,
            type = "description",
            name= " ",
        },
        stats = {
			order = 351,
            type = "execute",
            name = L["View Session Info"],
            desc = L["Displays in the chat window the current RepBuddy session information."],
            func = "SessionInfo"
        },
        settings = {
			order = 352,
            type = "execute",
            name = L["View Settings"],
            desc = L["Displays in the chat window the current RepBuddy settings."],
            func = "RepBuddySettings"
        },
        samplegroup = {
            type = "group",
			order = 400,
			guiInline = true,
            cmdHidden = true,
			name = L["Sample Chatframe Output:"],
            args = {
                sample = {
			        order = 402,
			        type = "description",
			        name = function(info) --Create sample output, refreshes on option change
                        local plusminus = "+"
	                    local neutraltxt = "|cfffdd017"
	                    local friendlytxt = "|cff4cc417"
	                    local change = 200
                    	local name = L["Therazane"]
                    	local nextstanding = L["Friendly"]
                    	local currentstanding = L["Neutral"]
                    	local currentstanding_amt = "2,400"
                    	local maxstanding_amt = "3,000"
                    	local remaining = 600
	                    local repetitions = 3
	                    local timetonext = TimeText(792)
	                    local reptotal = "29,384"
	                    local session_timetxt = TimeText(4336)
	                    local RPH = "2,337"
                        local percent_formatted = "80%"
                        local blizzrep = "|cff8884ce"..L["Reputation with Therazane increased by 200."].."|r\n"
                        local sessionstats = format(L["%s%s%d%s %s%s - %s%s%s until %s%s%s %s(%s / %s)%s"], 
												greentxt,plusminus,change,neutraltxt,name,nocolortxt,yellowtxt,remaining,nocolortxt,
												friendlytxt,nextstanding,nocolortxt,neutraltxt,currentstanding_amt,maxstanding_amt,nocolortxt)
	                    local morestats= format(L["    [~%s; %s%s%s repetitions remaining]%s - %s %s%s"], 
											timetonext,yellowtxt,repetitions,nocolortxt,neutraltxt,percent_formatted,currentstanding,nocolortxt)
	                    local sessionrep = format(L["    %sSession Total:%s %s%s%s %s"], 
											orangetxt,nocolortxt,greentxt,reptotal,nocolortxt,name)
	                    local sessiontime= format(L["    %sSession Time:%s %s"], 
											orangetxt,nocolortxt,session_timetxt)
	                    local sessionrph = format(L["    %sSession Rate:%s %s%s%s RPH"], 
											orangetxt,nocolortxt,yellowtxt,RPH,nocolortxt)						
	                    local statusheading = yellowtxt .. "Sample Output:" .. nocolortxt
                        local sampleoutput = ""
	                    if not RepBuddy.db.profile.hideBlizzRep then
                            sampleoutput = sampleoutput..blizzrep
                        end
                        if  (RepBuddy.db.profile.showInChat == true) then
		                    if (RepBuddy.db.profile.repinfo_verbose_display == true) then
			                    sampleoutput = sampleoutput..sessionstats.."\n"..morestats
		                    else
			                    sampleoutput = sampleoutput..sessionstats
                            end
                        end
		                if (RepBuddy.db.profile.sessionrep_display == true) then
			                sampleoutput = sampleoutput.."\n"..sessionrep
		                end
		                if (RepBuddy.db.profile.sessiontime_display == true) then
			                sampleoutput = sampleoutput.."\n"..sessiontime
		                end
		                if (RepBuddy.db.profile.sessionrph_display == true) then
			                sampleoutput = sampleoutput.."\n"..sessionrph
		                end
                        return sampleoutput 
                   end,
		        },
            },
        },
    },
}

----------------------------------------------------------------------------------------------------------------
-- Glamour Options
----------------------------------------------------------------------------------------------------------------
RepBuddyOptionsGlamour = {
    name = "Glamour",
    handler = RepBuddy,
    type = 'group',
  	args = {	
		descGlamour1 = {
			order = 1,
			type = "description",
			name = L["Glamour has been detected!"].."\n",
		},
		
		showGlamour = {	
            order = 2,
			width = full,
			type = "toggle",
            name = "Use Glamour",
            desc = L["Enhance Faction Changes with Glamour Alerts"],
            get = function() return RepBuddy.db.profile.useGlamour end,
            set = "ToggleUseGlamour"
        },
		
		descGlamour2 = {
			order = 3,
			type = "description",
			name = "\n\n".."Glamour simulates the Blizzard Achievement Notification when certain events occur.".."\n\n"..L["RepBuddy can display this effect when a faction change occurs (i.e. increase from revered to exalted with a specific faction.)"].."\n\n"..L["You can view a sample Glamour Alert using the controls below:"].."\n",
		},
		
		testName = {
			order = 4,
            type = "input",
			name = L["Enter Faction Name:"],
			desc = L["Enter any name to display it as the Faction Name in the Sample Glamour Alert."],
			get = function() return RepBuddy.db.profile.glamourTestName end,
			set = function(info, value) RepBuddy.db.profile.glamourTestName = value end,
        },	
		
		testStanding = {
			type = "select",
			order = 5,
			name = L["Choose Faction Standing:"],
			desc = L["Glamour Alerts change depending on standing, choose a standing and press 'Sample Glamour Alert' to see what it looks like."],
			values = {
				[1] = L["Hated"],
				[2] = L["Hostile"],
				[3] = L["Unfriendly"],
				[4] = L["Neutral"],
				[5] = L["Friendly"],
				[6] = L["Honored"],
				[7] = L["Revered"],
				[8] = L["Exalted"],
			},
			get = function() return RepBuddy.db.profile.glamourTestStanding end,
            set = function(info, value) RepBuddy.db.profile.glamourTestStanding = value end,
		},
		
		testGlamour = {
			order = 6,
            type = "execute",
            name = L["Sample Glamour Alert"],
            desc = L["Show Sample Glamour Alert."],
            func = "GlamourAlertSample"
        },

	},
}

----------------------------------------------------------------------------------------------------------------
-- Help Options
----------------------------------------------------------------------------------------------------------------
RepBuddyOptionsHelp = {
	type = "group",
	order = 1,
	name = "Help",
	args = {
		header = {
		    type = "header",
		    name = L["Frequently Asked Questions"],
		    order = 0,
	    },
	    text = {
			order = 1,
			type = "description",
			name = L["FAQ_BODY"],
		},
	},
}
