﻿--[[
********************************************************************************
RepBuddy
v1.30
July 4, 2011

Author: Phockre (Phockre @ Darrowmere US Alliance)
********************************************************************************

Description:
	RepBuddy provides detailed reputation information when reputation updates occur
    and stores reputation information per session.

Features:
	- Output detailed reputation information to the chatframe:
        * Reputation earned per update, amount required for next standing, current standing amount.
        * Estimated time left until next standing, number of repetitions at curret rate, percentage complete (OPTIONAL)
        * Total reputation gained/lost for the faction. (OPTIONAL)
        * Estimated Rep/Hour (RPH) (OPTIONAL)
        * Time elapsed in session (OPTIONAL)
    - Hide Blizzard reputation updates if desired.
    - Automatically change reputation tracker bar to reflect current reputation if desired.
    - Full GUI Config:
        * Enable/Disable in-game
        * Enable/Disable in-game.
        * Sample output based on config
        * View session stats in chatframe
        * View current settings in chatframe
        * Open Blizzard reputation frame
        * Reset session
        * Suggest reset when entering instance
    - Remembers settings via database, profile.
    - Detects and uses Glamour if installed/desired to provide blizzlike "Achievement" alert on faction standing change.
	- Full in-game help file and FAQ

Download:
	The latest version of Routes is always available on
	- http://www.wowace.com/projects/repbuddy/
	- http://wow.curse.com/downloads/wow-addons/details/repbuddy.aspx
	- http://www.wowinterface.com/downloads/info11401-Routes.html

Localization:
	You can contribute by updating/adding localizations using the system on
	- http://www.wowace.com/projects/repbuddy/localization/

]]

----------------------------------------------------------------------------------------------------------------
-- Addon Declarations
----------------------------------------------------------------------------------------------------------------
RepBuddy = LibStub("AceAddon-3.0"):NewAddon("RepBuddy", "AceConsole-3.0", "AceEvent-3.0")
local RepBuddy = _G.RepBuddy
local L = LibStub("AceLocale-3.0"):GetLocale("RepBuddy", true)
RepBuddyLDB = LibStub("LibDataBroker-1.1", true)
RepBuddy.version = GetAddOnMetadata("RepBuddy", "Version")
RepBuddy.versionstring = "RepBuddy v"..GetAddOnMetadata("RepBuddy", "Version")
_G["RepBuddy"] = RepBuddy

----------------------------------------------------------------------------------------------------------------
-- Database Defaults
----------------------------------------------------------------------------------------------------------------
local RepBuddy_Defaults = {
    profile = {
        turnedOn = true,
        showInChat = true,
        showGUI = false,
		repinfo_verbose_display = true,
		sessionrep_display = true,
		sessiontime_display = false,
		sessionrph_display = true,
		changeBar = false,
		useGlamour = true,
		glamourTestStanding = 4,
		glamourTestName = "Sample Reputation",
		hideBlizzRep = false,
		expandHeaders = false,
        outputFrame = 1,
        suggestReset = false,
        chatFrameID = 1,
        minimapIcon = {
					hide = false,
					},
    },
}

----------------------------------------------------------------------------------------------------------------
-- Global Variables
----------------------------------------------------------------------------------------------------------------
REPBUDDY_VERSION = GetAddOnMetadata("RepBuddy", "Version") or "Unknown Version"
REPBUDDY_TITLE = GetAddOnMetadata("RepBuddy", "Title") or "Unknown Title"
whitetxt, yellowtxt, orangetxt, redtxt, greentxt, bluetxt, nocolortxt = "|cffffffff","|cffffff00","|cffff7831","|cffff0000","|cff00ff00","|cff00FFFF", "|r"
GetFactionInfo = GetFactionInfo
factionVars = {}
session_start = GetTime()
standingmax = 8
standingmin = 1
freshsession = true
buildnewfactions = 0
lastinstance = ""
GlamourRepColors = { 
		[1] = {r = 0.54, g = 0.11, b = 0.07},
 		[2] = {r = 0.65, g = 0.30, b = 0.10},
		[3] = {r = 0.70, g = 0.48, b = 0.11},
		[4] = {r = 0.67, g = 0.55, b = 0.11},
		[5] = {r = 0.49, g = 0.49, b = 0.00},
		[6] = {r = 0.34, g = 0.47, b = 0.00},
		[7] = {r = 0.14, g = 0.48, b = 0.00},
		[8] = {r = 0.01, g = 0.49, b = 0.42} 
};

----------------------------------------------------------------------------------------------------------------
-- Local Variables
----------------------------------------------------------------------------------------------------------------
local fac_change = 0
local factions = 0

----------------------------------------------------------------------------------------------------------------
-- Initialize Addon (Ace3)
----------------------------------------------------------------------------------------------------------------
function RepBuddy:OnInitialize()
	
    --Initialize database
	RepBuddy.db = LibStub("AceDB-3.0"):New("RepBuddy", RepBuddy_Defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateProfile")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateProfile")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateProfile")
	
	--Turn RepBuddy on or off based on db (default is on)
	self:SetEnabledState(self.db.profile.turnedOn)
    
	--Check for Glamour, check version, set up variables, etc (code based on Titan Reputation)	
	if(IsAddOnLoaded("Glamour")) then
		self.CheckGlamour()
	end
    
    --Set Chatframe
    chatframe_rb = _G["ChatFrame"..RepBuddy.db.profile.chatFrameID]

    
	--Hide Blizz rep if enabled (default is false)
    local function filter()
		if (self.db.profile.hideBlizzRep == true) then
			return true
		else
			return false
		end
	end
    ChatFrame_AddMessageEventFilter("CHAT_MSG_COMBAT_FACTION_CHANGE", filter)
	
	--Register Options, Build Slash and GUI C0nfig Menu
    RepBuddyOptionsProfile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RepBuddy", RepBuddyOptionsMain)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("RepBuddy Profile", RepBuddyOptionsProfile)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("RepBuddy Glamour", RepBuddyOptionsGlamour)
    LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("RepBuddy Help", RepBuddyOptionsHelp)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RepBuddy", RepBuddy.versionstring)
	if(IsAddOnLoaded("Glamour")) then
		self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RepBuddy Glamour", "Glamour",RepBuddy.versionstring)
	end
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RepBuddy Profile", "Profile", RepBuddy.versionstring)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RepBuddy Help", "Help", RepBuddy.versionstring)
	self:RegisterChatCommand("rep", "ChatCommand")
	self:RegisterChatCommand("repbuddy", "ChatCommand")
	
    --Verify LDB library, create minimap icon control
    if RepBuddyLDB then
		createLDBMiniMap()
	end
    
    --Create reset confirm popup
    StaticPopupDialogs["SUGGEST_RESET"] = {
        text = L["Would you like to reset RepBuddy?"],
        button1 = L["Yes"],
        button2 = L["No"],
        OnAccept = function()
            RepBuddy:ResetStats(info, value)
        end,
        timeout = 0,
        whileDead = true,
        hideOnEscape = true,
    }

	--Print Welcome/Status
    if (self.db.profile.turnedOn == false) then
	    self:Print(RepBuddy.versionstring..L[" Loaded  - /rep or /repbuddy for options."])
        self:Print("|cffff0000"..L[" DISABLED"].."|r")
	else
        self:Print(RepBuddy.versionstring..L[" Loaded  - /rep or /repbuddy for options."])
	end	
end

----------------------------------------------------------------------------------------------------------------
-- Run when enabled (Ace3)
----------------------------------------------------------------------------------------------------------------
function RepBuddy:OnEnable()
	
    self:Print("|cff00ff00"..L[" ENABLED"].."|r")
    
    --Register events
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "DetectInstance")
    self:RegisterEvent("PLAYER_ALIVE", "ScanFactions")
    self:RegisterEvent("UPDATE_FACTION", "DisplayFactionData")
    self:RegisterEvent("GUILD_REP_UPDATED", "DisplayFactionData")
    self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE", "DisplayFactionData")
    -- Do initial scan
    self:ScanFactions()
end

---------------------------------------------------------------------------------------------------------------
-- Scan Factions and set up/update table for comparison
---------------------------------------------------------------------------------------------------------------
function RepBuddy:ScanFactions()
    local tempfac = GetNumFactions()
    --Fires first time and on reset
    if (tempfac ~= 0 and buildnewfactions == 0) then 
        if (GetNumFactions() == 0) then DEFAULT_CHAT_FRAME:AddMessage(L["Factions not yet available"]) end
        for i = 1, GetNumFactions() do
	        local name, _, standingID, _, _, barValue, _, _, isHeader, isCollapsed, hasRep = GetFactionInfo(i)
            if (not isHeader or hasRep) then
	    	    factionVars[name] = {}
	        	factionVars[name].Standing = standingID
	          	factionVars[name].Value = barValue
		        if (not factionVars[name].RepTotal) then
			    	factionVars[name].RepTotal = 0
		        end
            end
        end
		buildnewfactions = 1
		factions = tempfac
		self:ScanFactions()
	end
	
    --Fires if new faction added or headers expanded - essentially when "new" faction detected
    if (fac_change == 1) then
        if (GetNumFactions() == 0) then DEFAULT_CHAT_FRAME:AddMessage(L["Factions not yet available"]) end
        for i = 1, GetNumFactions() do
	        local name, _, standingID, _, _, barValue, _, _, isHeader, isCollapsed, hasRep = GetFactionInfo(i)
            if (not isHeader or hasRep) then
		        if (not factionVars[name]) then 
                    factionVars[name] = {}
                end
	        	factionVars[name].Standing = standingID
	    	    factionVars[name].Value = barValue
    	        if (not factionVars[name].RepTotal) then
				    factionVars[name].RepTotal = 0
	   	        end
	        end
        end 
		factions = tempfac
        fac_change = 0
	end

end

---------------------------------------------------------------------------------------------------------------
-- Run when faction updates occur (game event UPDATE_FACTION)
--------------------------------------------------------------------------------------------------------------
function RepBuddy:DisplayFactionData(event)
    --Rescan factions if guild rep update fires - band-aid for weird issue with missing guild rep updates
    if (event == "GUILD_REP_UPDATED") then
          fac_change = 1
          self:ScanFactions()
    end
    --Check for new factions (also detects if headers expanded in rep window)
    local temp = GetNumFactions()
    local isGuild = false
    if (temp ~= factions) then
        fac_change = 1
        self:ScanFactions()
    end
    --Update session timer (used for rep/hr, etc)
	local session_time =  GetTime() -	session_start
	local session_timetxt = TimeText(floor(session_time))
    
    --Scan factions, update those that have changed, store info, ouput to chatframe
    for factionIndex = 1, GetNumFactions() do
		local repcolortxt, nextrepcolortxt, plusminustxt = whitetxt
		local plusminus
		local name, _, standingID, barMin, barMax, barValue, _, _, isHeader, isCollapsed, hasRep = GetFactionInfo(factionIndex)
        
        --Make sure not a header (isHeader) UNLESS Alliance Vanguard or Horde Expedition (hasRep)
        if (not isHeader or hasRep) and factionVars[name] then

			--Calculate rep change, set color based on standing
            local diff = barValue - factionVars[name].Value
			repcolortxt=TxtColor(standingID)
			

			--If there is a change, process it; otherwise, repeat loop for next faction
			if diff ~= 0 then
				
				--Change repuatation tracker bar if enabled
				if (self.db.profile.changeBar == true) then
					SetWatchedFactionIndex(factionIndex)
				end
				
				--Deal with standing change if occured

                if standingID ~= factionVars[name].Standing then
                    local newfaction = _G["FACTION_STANDING_LABEL" .. standingID]
				    repcolortxt = TxtColor(standingID)
				    local newstandingtext = L["New standing with "] .. bluetxt .. name .. nocolortxt .." is " .. repcolortxt .. newfaction .. nocolortxt .. "!"
				    chatframe_rb:AddMessage(newstandingtext)
					
				    --If Glamour is loaded and enabled then use it to simulate blizzlike achievement alert
				    if(IsAddOnLoaded("Glamour")) then
					    if (self.db.profile.useGlamour == true) then
                        local MyData = {}
                                MyData.Text = name.." - "..getglobal("FACTION_STANDING_LABEL"..standingID)
							    MyData.Icon = "Interface\\ICONS\\Achievement_Reputation_0"..standingID
							    local color = { }
							    color.r = GlamourRepColors [standingID].r
							    color.g = GlamourRepColors [standingID].g
							    color.b = GlamourRepColors [standingID].b
							    MyData.bTitle = L["Faction Standing Changed!"]
							    MyData.FrameStyle = "GuildAchievement"
							    MyData.BannerColor = color

							    GlamourShowAlert(400, MyData, green, red)
					    end
				    end
				end
				
				local remaining, nextstanding, plusminus
				
                --Do if rep increased
				if diff > 0 then
					plusminustxt = greentxt
					plusminus="+"
					remaining = barMax - barValue
					if standingID < standingmax then
						nextstanding = _G["FACTION_STANDING_LABEL" .. standingID + 1]
						nextrepcolortxt=TxtColor(standingID + 1)
					else
						nextstanding = L["Totally "] .. _G["FACTION_STANDING_LABEL" .. standingmax]
						nextrepcolortxt=TxtColor(8)
					end
				--Do if rep decreased
                else
					plusminustxt = redtxt
					plusminus="-"
						remaining = barValue - barMin
					if standingID > standingmin then
						nextstanding = _G["FACTION_STANDING_LABEL" .. standingID - 1]
						nextrepcolortxt=TxtColor(standingID - 1)
					else
						nextstanding = L["Totally "] .. _G["FACTION_STANDING_LABEL" .. standingmin]
						nextrepcolortxt=TxtColor(1)
					end
				end
                
                --Calculate statistics
				local rawchange = barValue - factionVars[name].Value
                local change = math.abs(rawchange)
				local currentstanding = _G["FACTION_STANDING_LABEL" .. standingID]
				local repetitions = math.ceil(remaining / change)
				local currentstanding_amt = abs((abs(barValue)-abs(barMin)))
				local maxstanding_amt = abs((abs(barMax)-abs(barMin)))
				local reptotal = factionVars[name].RepTotal + rawchange
				local timetonext = TimeText(floor(remaining / (math.abs(reptotal) / session_time)))
				local RPH = floor(math.abs(reptotal) / (session_time / 60 / 60))
				if (RPH < 1) then
					RPH = round(reptotal / (session_time / 60 / 60),2)
				end
                local percent
                if (standingID < 4) then
                    percent = 100-round((currentstanding_amt / maxstanding_amt)*100,1)
                else
                    percent = round((currentstanding_amt / maxstanding_amt)*100,1)
                end
			    local reptotaltxt = greentxt
                if (reptotal < 0) then
                    reptotaltxt = redtxt
                end
				
                --Make values over 1,000 pretty with comma separation
				RPH = comma_value(RPH)
				local repetitions_formatted = comma_value(repetitions)
				local reptotal_formatted = comma_value(reptotal)
                local remaining_formatted = comma_value(remaining)
                local currentstanding_amt_formatted = comma_value(currentstanding_amt)
                local maxstanding_amt_formatted = comma_value(maxstanding_amt)
                local percent_formatted = percent.."%"
				
				--Build frame outputs
	            local sessionstats= format(L["%s%s%d%s %s%s - %s%s%s until %s%s%s %s(%s / %s)%s"], 
						                plusminustxt,plusminus,change,repcolortxt,name,nocolortxt,yellowtxt,remaining_formatted,nocolortxt,
										nextrepcolortxt,nextstanding,nocolortxt,repcolortxt,currentstanding_amt_formatted,maxstanding_amt_formatted,nocolortxt)
				local morestats= format(L["    [~%s; %s%s%s repetitions remaining]%s - %s %s%s"], 
											timetonext,yellowtxt,repetitions_formatted,nocolortxt,repcolortxt,percent_formatted,currentstanding,nocolortxt)
				local sessionrep = format(L["    %sSession Total:%s %s%s%s %s"], 
											orangetxt,nocolortxt,reptotaltxt,reptotal_formatted,nocolortxt,name)
				local sessiontime= format(L["    %sSession Time:%s %s"], 
											orangetxt,nocolortxt,session_timetxt)
				local sessionrph = format(L["    %sSession Rate:%s %s%s%s RPH"], 
											orangetxt,nocolortxt,yellowtxt,RPH,nocolortxt)						
				
				--Output to chatframe based on options
				if (self.db.profile.repinfo_verbose_display == true) then
						chatframe_rb:AddMessage(sessionstats)
						chatframe_rb:AddMessage(morestats)
               else
						chatframe_rb:AddMessage(sessionstats)
				end
				if (self.db.profile.sessionrep_display == true) then
						chatframe_rb:AddMessage(sessionrep)
				end
				if (self.db.profile.sessiontime_display == true) then
						chatframe_rb:AddMessage(sessiontime)
				end
				if (self.db.profile.sessionrph_display == true) then
						chatframe_rb:AddMessage(sessionrph)
				end
                
                --Update stored info with new data
				factionVars[name].Value = barValue
				factionVars[name].Standing = standingID
				factionVars[name].RepTotal = reptotal
			end
		end
	end
end

----------------------------------------------------------------------------------------------------------------
-- Run when enabled (Ace3)
----------------------------------------------------------------------------------------------------------------
function RepBuddy:OnDisable()
   self:Print("|cffff0000"..L[" DISABLED"].."|r")
end
