﻿--[[
********************************************************************************
RepBuddy Support Functions
********************************************************************************
]]

----------------------------------------------------------------------------------------------------------------
-- Declarations
----------------------------------------------------------------------------------------------------------------
local RepBuddy = LibStub("AceAddon-3.0"):GetAddon("RepBuddy")
local L = LibStub("AceLocale-3.0"):GetLocale("RepBuddy", true)
local RepBuddyIcon = LibStub("LibDBIcon-1.0", true)

----------------------------------------------------------------------------------------------------------------
-- Utility Functions
----------------------------------------------------------------------------------------------------------------
function RepBuddy:DetectInstance()
    local inInstance, instanceType = IsInInstance()
    
    --Check if in instance, suggest reset if enabled
    if inInstance then
        local instance = GetInstanceInfo()
        if (RepBuddy.db.profile.suggestReset == true) then
            if (lastinstance ~= instance) then
                StaticPopup_Show ("SUGGEST_RESET")
                lastinstance = instance
            end
        end
    end
end

function TxtColor(standingid)
	
    -- Sets color of faction text
    local colorcode
	if     standingid == 1  then colorcode = "|cfff11b17" --hated
	elseif standingid == 2  then colorcode = "|cffF62817" --hostile
	elseif standingid == 3 	then colorcode = "|cfff88017" --unfriendly
	elseif standingid == 4  then colorcode = "|cfffdd017" --neutral
	elseif standingid == 5  then colorcode = "|cff348017" --friendly
	elseif standingid == 6  then colorcode = "|cff4cc417" --honored
	elseif standingid == 7  then colorcode = "|cff4cc417" --revered
	elseif standingid == 8  then colorcode = "|cff4cc417" --exalted
	else   colorcode = "|cffffffff"
	end
	return colorcode
end

function TimeText(rawtime)

	-- Converts raw time (seconds) to long form time (X hr X min X secs)
	local days = floor(rawtime/24/60/60); rawtime = mod(rawtime, 24*60*60);
	local hours = floor(rawtime/60/60); rawtime = mod(rawtime, 60*60);
	local minutes = floor(rawtime/60); rawtime = mod(rawtime, 60);
 	local seconds = rawtime;
  	local longtime = "";
	local yellowtxt, nocolortxt = "|cffffff00","|r"
  	if (days ~= 0) then
		if (days==1) then
			longtime = longtime..format("%s%d%s"..L[" day "], yellowtxt,days,nocolortxt);
		else
			longtime = longtime..format("%s%d%s"..L[" days "], yellowtxt,days,nocolortxt);
		end
  	end
  	if (days ~= 0 or hours ~= 0) then
		if (hours==1) then
   			longtime = longtime..format("%s%d%s"..L[" hr "], yellowtxt,hours,nocolortxt);
		else
			longtime = longtime..format("%s%d%s"..L[" hrs "], yellowtxt,hours,nocolortxt);
		end
  	end
  	if (days ~= 0 or hours ~= 0 or minutes ~= 0) then
		if (minutes == 1) then
    		longtime = longtime..format("%s%d%s"..L[" min "], yellowtxt,minutes,nocolortxt);
		else
			longtime = longtime..format("%s%d%s"..L[" mins "], yellowtxt,minutes,nocolortxt);
		end
  	end  
	if (seconds ==1) then
  		longtime = longtime..format("%s%d%s"..L[" sec"], yellowtxt,seconds,nocolortxt);
	else
		longtime = longtime..format("%s%d%s"..L[" secs"], yellowtxt,seconds,nocolortxt);
	end
  
  	return longtime;

end

function round(val, decimal)

	-- Round number to desired number of decimal places (courtesy of http://lua-users.org/wiki/FormattingNumbers)
    if (decimal) then
        return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
    else
        return math.floor(val+0.5)
    end
end

function comma_value(amount)

	-- Adds commas to separate thousands (courtesy of http://lua-users.org/wiki/FormattingNumbers)
  	local formatted = amount
  	while true do  
    	formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
    	if (k==0) then
      		break
    	end
  	end
  	return formatted
end


function createLDBMiniMap()
    
    -- Builds and creates minimap icon
	local L_BT_LEFT = "|cffff7831"..L["Left-Click"].."|r"..L[" to open the options menu."]
	local L_BT_RIGHT = "|cffff7831"..L["Right-Click"].."|r"..L[" to open reputation window."]
	local L_BT_ALTRIGHT = "|cffff7831"..L["Alt-Right-Click"].."|r"..L[" to view stats."]

	local RepBuddyLDBObj = LibStub("LibDataBroker-1.1"):NewDataObject("RepBuddy", {
		type = "launcher",
		label = "RepBuddy",
		OnClick = function(_, msg)
			if msg == "LeftButton" then
    			if ( InterfaceOptionsFrame:IsShown() ) then
        			InterfaceOptionsFrame:Hide();
    			else
       				 InterfaceOptionsFrame_OpenToCategory(RepBuddy.versionstring)
   				end	
			elseif msg == "RightButton" then
				if IsAltKeyDown() then
					RepBuddy:SessionInfo()
				else
					ToggleCharacter("ReputationFrame")
				end
			end
		end,
		icon = "Interface\\Icons\\Achievement_Reputation_06",
		
		OnTooltipShow = function(tooltip)
			if not tooltip or not tooltip.AddLine then return end
			tooltip:AddLine("RepBuddy")
			tooltip:AddLine(L_BT_LEFT)
			tooltip:AddLine(L_BT_RIGHT)
			tooltip:AddLine(L_BT_ALTRIGHT)
		end,
	})
	if RepBuddyIcon then
		RepBuddyIcon:Register("RepBuddy", RepBuddyLDBObj, RepBuddy.db.profile.minimapIcon)
	end
end


function RepBuddy:CheckGlamour(info)
    
    -- Check if Glamour is loaded and verify version
    if(IsAddOnLoaded("Glamour")) then
		local min_version = 1.4;
		local major, minor, _ = strsplit(".", Glamour_VERSION);
		local glam_ver = tonumber(major.."."..minor);
		if (glam_ver < min_version) then
			StaticPopupDialogs[L["! ! ! Glamour Outdated ! ! !"]] = {
				  text = L["An outdated version of Glamour has been detected. Running an older version of Glamour can have undesired effects on Glamour enabled addons."].."\n\n"..
			  		L["Reporting Addon:"].."\n"..REPBUDDY_TITLE.." v"..REPBUDDY_VERSION.."\n\n"..
					L["Glamour Version Detected: "]..Glamour_VERSION.."\n" ..L["Glamour Version Required: "]..min_version.."\n\n",
				  button1 = "Ok",
				  timeout = 0,
				  whileDead = true,
				  hideOnEscape = true,
			}
			StaticPopup_Show (L["! ! ! Glamour Outdated ! ! !"]);
		else
			RepBuddy:Print(L["Glamour Detected"])
		end
	end
end


function RepBuddy:ChatCommand(input)
    
    -- Opens GUI Options on slash with no arguements
     if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(RepBuddy.versionstring)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(RepBuddy, "rep", "RepBuddy", input:trim() ~= "help" and input or "")
    end
end

function RepBuddy:UpdateProfile()
    --Fires when profile changed, deleted etc.
    self:RepBuddySettings()
end

----------------------------------------------------------------------------------------------------------------
-- General Option Functions
----------------------------------------------------------------------------------------------------------------
function RepBuddy:ToggleTurnedOn(info, value)
    if value == true then
		self:Enable()
	else
		self:Disable()
	end
	self.db.profile.turnedOn = value	
end

function RepBuddy:ToggleMiniMap(info, value)
	self.db.profile.minimapIcon.hide = not value
	if self.db.profile.minimapIcon.hide then
		RepBuddyIcon:Hide("RepBuddy")
        self:Print(L["Show Minimap Icon"].."|cffff0000"..L[" DISABLED"].."|r")
	else
		RepBuddyIcon:Show("RepBuddy")
        self:Print(L["Show Minimap Icon"].."|cff00ff00"..L[" ENABLED"].."|r")
	end
end

function RepBuddy:ToggleUpdateBar(info, value)
    self.db.profile.changeBar = value
	if (self.db.profile.changeBar == true) then
		self:Print(L["Update Reputation Tracker Bar"].."|cff00ff00"..L[" ENABLED"].."|r")
	else
		self:Print(L["Update Reputation Tracker Bar"].."|cffff0000"..L[" DISABLED"].."|r")
	end
end

function RepBuddy:ToggleSuggestReset(info, value)
    self.db.profile.suggestReset = value
	if (self.db.profile.suggestReset == true) then
		self:Print(L["Suggest Reset When Entering Instance"].."|cff00ff00"..L[" ENABLED"].."|r")
	else
		self:Print(L["Suggest Reset When Entering Instance"].."|cffff0000"..L[" DISABLED"].."|r")
	end
end

function RepBuddy:ResetStats(info, value)
    
    --Resest session (timer, blank data container)
    session_start = GetTime()
    buildnewfactions = 0
    self:ScanFactions()
	self:Print("|cffff0000"..L[" SESSION RESET"].."|r")
	RepBuddy:RepBuddySettings()
end

----------------------------------------------------------------------------------------------------------------
-- General Option Functions
----------------------------------------------------------------------------------------------------------------
function RepBuddy:ToggleXtraStats(info, value)
    self.db.profile.repinfo_verbose_display = value
	if (self.db.profile.repinfo_verbose_display == true) then
		self:Print(L["Display Extra Stats"].."|cff00ff00"..L[" ENABLED"].."|r")
	else
		self:Print(L["Display Extra Stats"].."|cffff0000"..L[" DISABLED"].."|r")
	end
end

function RepBuddy:ToggleTotalDisplay(info, value)
    self.db.profile.sessionrep_display = value
	if (self.db.profile.sessionrep_display == true) then
		self:Print(L["Display Session Reputation"].."|cff00ff00"..L[" ENABLED"].."|r")
	else
		self:Print(L["Display Session Reputation"].."|cffff0000"..L[" DISABLED"].."|r")
	end
end

function RepBuddy:ToggleRPHDisplay(info, value)
    self.db.profile.sessionrph_display = value
	if (self.db.profile.sessionrph_display == true) then
		self:Print(L["Display Rep/Hour (RPH)"].."|cff00ff00"..L[" ENABLED"].."|r")
	else
		self:Print(L["Display Rep/Hour (RPH)"].."|cffff0000"..L[" DISABLED"].."|r")
	end
end

function RepBuddy:ToggleElapsedDisplay(info, value)
    self.db.profile.sessiontime_display = value
	if (self.db.profile.sessiontime_display == true) then
		self:Print(L["Display Session Time"].."|cff00ff00"..L[" ENABLED"].."|r")
	else
		self:Print(L["Display Session Time"].."|cffff0000"..L[" DISABLED"].."|r")
	end
end

function RepBuddy:ToggleHideBlizzRep(info, value)
    self.db.profile.hideBlizzRep = value
	if (self.db.profile.hideBlizzRep == true) then
		self:Print(L["Hide Blizzard Rep Notifications"].."|cff00ff00"..L[" ENABLED"].."|r")
	else
		self:Print(L["Hide Blizzard Rep Notifications"].."|cffff0000"..L[" DISABLED"].."|r")
	end
end

function RepBuddy:SessionInfo(info)
    local nodata = true
	chatframe_rb:AddMessage(yellowtxt..L["RepBuddy Session Info:"]..nocolortxt)
    local repcolortxt, nextrepcolortxt = whitetxt
	if (GetNumFactions() == 0) then DEFAULT_CHAT_FRAME:AddMessage(L["Factions not yet available"]) end
	for i = 1, GetNumFactions() do
		local name, _, standingID, barMin, barMax, barValue, _, _, isHeader, isCollapsed, hasRep = GetFactionInfo(i)
		repcolortxt=TxtColor(standingID)
		local currentstanding = _G["FACTION_STANDING_LABEL" .. standingID]
		local currentstanding_amt = abs((abs(barValue)-abs(barMin)))
		local maxstanding_amt = abs((abs(barMax)-abs(barMin)))
        local currentstanding_amt_formatted = comma_value(currentstanding_amt)
        local maxstanding_amt_formatted = comma_value(maxstanding_amt)
        local percent
        if (standingID < 4) then
            percent = 100-round((currentstanding_amt / maxstanding_amt)*100,1)
        else
            percent = round((currentstanding_amt / maxstanding_amt)*100,1)
        end
        local percent_formatted = percent.."%"
		if (not isHeader or hasRep) then
			if (not IsFactionInactive(i) and factionVars[name].RepTotal) then
				local total = factionVars[name].RepTotal + 0
				if (total ~= 0) then
                    nodata = false
                    local total_formatted = comma_value(total)
                    local totaltxt = greentxt
                    if (total < 0) then
                        totaltxt = redtxt
                    end
					local sessionstats= format("%s%s%s: %s%s%s rep this session - %s(%s / %s) - %s %s%s", 
												repcolortxt,name,nocolortxt,totaltxt,total_formatted,nocolortxt,
												repcolortxt,currentstanding_amt_formatted,maxstanding_amt_formatted,percent_formatted,currentstanding,nocolortxt)
                    chatframe_rb:AddMessage(sessionstats)
				end
			end
		end
	end
    if (nodata == true) then
        chatframe_rb:AddMessage(redtxt..L["   No reputation data to report."]..nocolortxt)
    end
end

function RepBuddy:RepBuddySettings(info, value)
	
    --Dump settings info to chatframe
	local session_time =  GetTime() -	session_start
	local session_timetxt = TimeText(floor(session_time))
    local name = GetChatWindowInfo(self.db.profile.chatFrameID)
	if (self.db.profile.turnedOn == false) then
		self:Print("|cffff0000"..L[" DISABLED"].."|r")
	else
        self:Print("Current Session Timer: "..session_timetxt)
        self:Print(L["Output to "].."|cff00ff00"..name.." (ChatFrame"..self.db.profile.chatFrameID..")|r")
        if (self.db.profile.minimapIcon.hide == false) then
			self:Print(L["Show Minimap Icon"].."|cff00ff00"..L[" ENABLED"].."|r")
		else
			self:Print(L["Show Minimap Icon"].."|cffff0000"..L[" DISABLED"].."|r")
		end
        if (self.db.profile.changeBar == true) then
			self:Print(L["Update Reputation Tracker Bar"].."|cff00ff00"..L[" ENABLED"].."|r")
		else
			self:Print(L["Update Reputation Tracker Bar"].."|cffff0000"..L[" DISABLED"].."|r")
		end    
        if (self.db.profile.suggestReset  == true) then
			self:Print(L["Suggest Reset When Entering Instance"].."|cff00ff00"..L[" ENABLED"].."|r")
		else
			self:Print(L["Suggest Reset When Entering Instance"].."|cffff0000"..L[" DISABLED"].."|r")
		end
		if (self.db.profile.repinfo_verbose_display == true) then
			self:Print(L["Display Extra Stats"].."|cff00ff00"..L[" ENABLED"].."|r")
		else
			self:Print(L["Display Extra Stats"].."|cffff0000"..L[" DISABLED"].."|r")
		end
		if (self.db.profile.sessionrep_display == true) then
			self:Print(L["Display Session Reputation"].."|cff00ff00"..L[" ENABLED"].."|r")
		else
			self:Print(L["Display Session Reputation"].."|cffff0000"..L[" DISABLED"].."|r")
		end
		if (self.db.profile.sessionrph_display == true) then
			self:Print(L["Display Rep/Hour (RPH)"].."|cff00ff00"..L[" ENABLED"].."|r")
		else
			self:Print(L["Display Rep/Hour (RPH)"].."|cffff0000"..L[" DISABLED"].."|r")
		end
		if (self.db.profile.sessiontime_display == true) then
			self:Print(L["Display Session Time"].."|cff00ff00"..L[" ENABLED"].."|r")
		else
			self:Print(L["Display Session Time"].."|cffff0000"..L[" DISABLED"].."|r")
		end
		if (self.db.profile.hideBlizzRep == true) then
			self:Print(L["Hide Blizzard Rep Notifications"].."|cff00ff00"..L[" ENABLED"].."|r")
		else
			self:Print(L["Hide Blizzard Rep Notifications"].."|cffff0000"..L[" DISABLED"].."|r")
		end
		if(IsAddOnLoaded("Glamour")) then
				if (self.db.profile.useGlamour == true) then
					self:Print(L["Use Glamour"].."|cff00ff00"..L[" ENABLED"].."|r")
				else
					self:Print(L["Use Glamour"].."|cffff0000"..L[" DISABLED"].."|r")
				end
		end
	end
end

----------------------------------------------------------------------------------------------------------------
-- Glamour Functions
----------------------------------------------------------------------------------------------------------------
function RepBuddy:ToggleUseGlamour(info, value)
    self.db.profile.useGlamour = value
	if (self.db.profile.useGlamour == true) then
		self:Print(L["Use Glamour"].."|cff00ff00"..L[" ENABLED"].."|r")
	else
		self:Print(L["Use Glamour"].."|cffff0000"..L[" DISABLED"].."|r")
	end
end

function RepBuddy:GlamourAlertSample()
    local testname = self.db.profile.glamourTestName
    local standingId = self.db.profile.glamourTestStanding
	local MyData = {}
	MyData.Text = testname.." - "..getglobal("FACTION_STANDING_LABEL"..standingId)
	MyData.Icon = "Interface\\ICONS\\Achievement_Reputation_0"..standingId
	local color = { }
	color.r = GlamourRepColors [standingId].r
	color.g = GlamourRepColors [standingId].g
	color.b = GlamourRepColors [standingId].b
	MyData.bTitle = L["Faction Standing Changed!"]
	MyData.FrameStyle = "GuildAchievement"
	MyData.BannerColor = color
    GlamourShowAlert(400, MyData, green, red)
    end


